#ifndef INC_ERRORS_H
#define INC_ERRORS_H

#include <limits.h>
#include <stdint.h>
#include <strings.h>

#define ERROR_TIME_THRESH 150 // ms

typedef enum {
  SEK_OVERTEMP,
  SEK_UNDERTEMP,
  SEK_OVERVOLT,
  SEK_UNDERVOLT,
  SEK_TOO_FEW_TEMPS,
  SEK_OPENWIRE,
  SEK_EEPROM_ERR,
  SEK_INTERNAL_BMS_TIMEOUT,
  SEK_INTERNAL_BMS_CHECKSUM_FAIL,
  SEK_INTERNAL_BMS_OVERTEMP,
  SEK_INTERNAL_BMS_FAULT,
  SEK_1WIRE_ERR,
  SEK_DS18B20_TIMEOUT,
  NUM_ERROR_KINDS
} SlaveErrorKind;

typedef struct {
  uint8_t data[4];
  uint32_t errors_since;
} SlaveErrorData;

extern uint32_t error_sources; // Bitfield of error sources
extern SlaveErrorData error_data[NUM_ERROR_KINDS];

static_assert(sizeof(error_sources) * CHAR_BIT >= NUM_ERROR_KINDS,
              "error_sources is too small to hold all error sources");

void set_error_source(SlaveErrorKind source);
void clear_error_source(SlaveErrorKind source);

#endif // INC_ERRORS_H
